==> z-web-support-0.1.6-sources.jar: cc/zuv/web/support/aspect/SysLogAspect.java <==
/*
 * Id$: zuv-cloud:z-web-support:cc.zuv.web.support.aspect.SysLogAspect:20181226101928
 *
 * SysLogAspect.java
 * Copyright (c) 2002-2020 Luther Inc.
 * http://zuv.cc
 * All rights reserved.
 */

package cc.zuv.web.support.aspect;

import cc.zuv.lang.StringUtils;
import cc.zuv.utility.KeyGenUtils;
import cc.zuv.web.support.annotation.SysLogRender;
import cc.zuv.web.support.enums.SysLogLevel;
import cc.zuv.web.support.enums.SysLogModule;
import com.google.gson.Gson;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.*;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.http.MediaType;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.lang.reflect.Method;

/**
 * 系统日志切面处理
 *
 * @author			Kama Luther
 * @version			0.1
 * @since           0.1
 * @create.date     2014-2-11 下午02:19:48
 * @modify.date     2014-2-11 下午02:19:48
 */
@Slf4j
public abstract class SysLogAspect
{

    //-----------------------------------------------------------------------------------------

	@Pointcut("@annotation(cc.zuv.web.support.annotation.SysLogRender)")
	public void logPointCut()
    {

	}

	@Around("logPointCut()")
	public Object around(ProceedingJoinPoint point) throws Throwable
    {
        //开始时间
		long beginTime = System.currentTimeMillis();

		//执行方法
		Object result = point.proceed();

		//执行时长(毫秒)
		long elapse = System.currentTimeMillis() - beginTime;

		//保存日志
		saveSysLog(point, elapse);

		return result;
	}

    //-----------------------------------------------------------------------------------------

	private void saveSysLog(ProceedingJoinPoint joinPoint, long elapse)
    {
        //
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = attributes.getRequest();

        //
		MethodSignature signature = (MethodSignature) joinPoint.getSignature();
		Method method = signature.getMethod();
        SysLogRender syslogrender = method.getAnnotation(SysLogRender.class);

        if(syslogrender==null) return;

        //注解的内容
        SysLogLevel     _level      = syslogrender.level();
        SysLogModule    _module     = syslogrender.module();
        String          _title      = syslogrender.value();
        String          _levelstr   = syslogrender.levelstr();
        String          _modulestr  = syslogrender.modulestr();

		//请求的方法
		String cname = joinPoint.getTarget().getClass().getName();
		String mname = signature.getName();
        String _method = cname + "." + mname + "()";

		//请求的参数
        String _params = null;
        if (request.getContentType()==null||
            MediaType.APPLICATION_FORM_URLENCODED_VALUE.contains(request.getContentType())
            || MediaType.APPLICATION_JSON_UTF8_VALUE.contains(request.getContentType()))
        {
            Object[] args = joinPoint.getArgs();
            if(args!=null && args.length>0)
            {
                _params = new Gson().toJson(args);
            }
        }

        //请求的地址
        String _hostip = request.getRemoteHost();
        String _requri = request.getRequestURI();
        String _reqmtd = request.getMethod();

        if(StringUtils.NotEmpty(_levelstr) && StringUtils.NotEmpty(_modulestr))
        {
            saveSysLog(_levelstr, _modulestr, _title, _method, _params, _hostip, _requri, _reqmtd, elapse);
        }
        else
        {
            saveSysLog(_level, _module, _title, _method, _params, _hostip, _requri, _reqmtd, elapse);
        }
	}

    protected abstract void saveSysLog(SysLogLevel level, SysLogModule module, String title,
                                       String method, String params, String hostip, String requri, String reqmtd, long elapse);

    protected abstract void saveSysLog(String level, String module, String title,
                                       String method, String params, String hostip, String requri, String reqmtd, long elapse);

    //-----------------------------------------------------------------------------------------

    private static final String REQUEST_ID = "$request_id";

    @Before("logPointCut()")
    public void doBefore(JoinPoint joinPoint) throws Throwable
    {
        //
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = attributes.getRequest();

        //
        String requestid = KeyGenUtils.uuid();
        request.setAttribute(REQUEST_ID, requestid);

        //
        String sessionid = request.getSession().getId();
        String requesturi = request.getRequestURI();

        //
        log.info("request: id={}, session= {}, requesturi={}", requestid, sessionid, requesturi);
    }

    @AfterReturning(returning = "ret", pointcut = "logPointCut()")
    public void doAfterReturning(Object ret) throws Throwable
    {
        //
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = attributes.getRequest();

        //
        String requestid = (String) request.getAttribute(REQUEST_ID);
        String sessionid = request.getSession().getId();
        String retrunval = new Gson().toJson(ret);

        //
        log.info("response: id={}, session= {}, ret={}", requestid, sessionid, retrunval);
    }

    @AfterThrowing(pointcut = "logPointCut()", throwing = "e")
    public void doAfterThrowing(Exception e)
    {
        //
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = attributes.getRequest();

        //
        String requestid = (String) request.getAttribute(REQUEST_ID);
        String sessionid = request.getSession().getId();
        String message = e.getMessage();

        //
        log.info("throwing: id={}, session= {}, message={}", requestid, sessionid, message);
    }

    //-----------------------------------------------------------------------------------------


    //-----------------------------------------------------------------------------------------


}

==> z-web-support-0.1.9-sources.jar: cc/zuv/web/support/aspect/SysLogAspect.java <==
/*
 * Id$: zuv-cloud:z-web-support:cc.zuv.web.support.aspect.SysLogAspect:20181226101928
 *
 * SysLogAspect.java
 * Copyright (c) 2002-2020 Luther Inc.
 * http://zuv.cc
 * All rights reserved.
 */

package cc.zuv.web.support.aspect;

import cc.zuv.lang.StringUtils;
import cc.zuv.utility.KeyGenUtils;
import cc.zuv.web.support.annotation.SysLogRender;
import cc.zuv.web.support.enums.SysLogLevel;
import cc.zuv.web.support.enums.SysLogModule;
import com.google.gson.Gson;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.*;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.http.MediaType;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import java.lang.reflect.Method;

/**
 * 系统日志切面处理
 *
 * @author			Kama Luther
 * @version			0.1
 * @since           0.1
 * @create.date     2014-2-11 下午02:19:48
 * @modify.date     2014-2-11 下午02:19:48
 */
@Slf4j
public abstract class SysLogAspect
{

    //-----------------------------------------------------------------------------------------

	@Pointcut("@annotation(cc.zuv.web.support.annotation.SysLogRender)")
	public void logPointCut()
    {

	}

	@Around("logPointCut()")
	public Object around(ProceedingJoinPoint point) throws Throwable
    {
        //开始时间
		long beginTime = System.currentTimeMillis();

		//执行方法
		Object result = point.proceed();

		//执行时长(毫秒)
		long elapse = System.currentTimeMillis() - beginTime;

		//保存日志
		saveSysLog(point, elapse);

		return result;
	}

    //-----------------------------------------------------------------------------------------

	private void saveSysLog(ProceedingJoinPoint joinPoint, long elapse)
    {
        //
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = attributes.getRequest();

        //
		MethodSignature signature = (MethodSignature) joinPoint.getSignature();
		Method method = signature.getMethod();
        SysLogRender syslogrender = method.getAnnotation(SysLogRender.class);

        if(syslogrender==null) return;

        //注解的内容
        SysLogLevel     _level      = syslogrender.level();
        SysLogModule    _module     = syslogrender.module();
        String          _title      = syslogrender.value();
        String          _levelstr   = syslogrender.levelstr();
        String          _modulestr  = syslogrender.modulestr();

		//请求的方法
		String cname = joinPoint.getTarget().getClass().getName();
		String mname = signature.getName();
        String _method = cname + "." + mname + "()";

		//请求的参数
        String _params = null;
        if (request.getContentType()==null||
            MediaType.APPLICATION_FORM_URLENCODED_VALUE.contains(request.getContentType())
            || MediaType.APPLICATION_JSON_UTF8_VALUE.contains(request.getContentType()))
        {
            Object[] args = joinPoint.getArgs();
            if(args!=null && args.length>0)
            {
                boolean ignore = false;
                for(Object arg : args)
                {
                    if(arg instanceof ServletResponse || arg instanceof ServletRequest)
                    {
                        ignore = true;
                        break;
                    }
                }

                if(!ignore) _params = new Gson().toJson(args);
            }
        }

        //请求的地址
        String _hostip = request.getRemoteHost();
        String _requri = request.getRequestURI();
        String _reqmtd = request.getMethod();

        if(StringUtils.NotEmpty(_levelstr) && StringUtils.NotEmpty(_modulestr))
        {
            saveSysLog(_levelstr, _modulestr, _title, _method, _params, _hostip, _requri, _reqmtd, elapse);
        }
        else
        {
            saveSysLog(_level, _module, _title, _method, _params, _hostip, _requri, _reqmtd, elapse);
        }
	}

    protected abstract void saveSysLog(SysLogLevel level, SysLogModule module, String title,
                                       String method, String params, String hostip, String requri, String reqmtd, long elapse);

    protected abstract void saveSysLog(String level, String module, String title,
                                       String method, String params, String hostip, String requri, String reqmtd, long elapse);

    //-----------------------------------------------------------------------------------------

    private static final String REQUEST_ID = "$request_id";

    @Before("logPointCut()")
    public void doBefore(JoinPoint joinPoint) throws Throwable
    {
        //
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = attributes.getRequest();

        //
        String requestid = KeyGenUtils.uuid();
        request.setAttribute(REQUEST_ID, requestid);

        //
        String sessionid = request.getSession().getId();
        String requesturi = request.getRequestURI();

        //
        log.info("request: id={}, session= {}, requesturi={}", requestid, sessionid, requesturi);
    }

    @AfterReturning(returning = "ret", pointcut = "logPointCut()")
    public void doAfterReturning(Object ret) throws Throwable
    {
        //
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = attributes.getRequest();

        //
        String requestid = (String) request.getAttribute(REQUEST_ID);
        String sessionid = request.getSession().getId();
        String retrunval = new Gson().toJson(ret);

        //
        log.info("response: id={}, session= {}, ret={}", requestid, sessionid, retrunval);
    }

    @AfterThrowing(pointcut = "logPointCut()", throwing = "e")
    public void doAfterThrowing(Exception e)
    {
        //
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = attributes.getRequest();

        //
        String requestid = (String) request.getAttribute(REQUEST_ID);
        String sessionid = request.getSession().getId();
        String message = e.getMessage();

        //
        log.info("throwing: id={}, session= {}, message={}", requestid, sessionid, message);
    }

    //-----------------------------------------------------------------------------------------


    //-----------------------------------------------------------------------------------------


}
